;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
;;; An individual-based model of a Konik horse population, based on:
;;;;De Roos, A. M., Galic, N., Heesterbeek, H., How resource competition shapes individual life history for nonplastic growth: Ungulates in seasonal food environments. Ecology 2009, 90, (4), 945-960.
;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
;;; Script: Nika Galic (ngalic2@unl.edu)
;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
;;; for information on model assumptions and description, see ODD (separate document)
;;; for things to try out with the model, see the Info tab
;;; year starts with 1st April - this is when all foals are born and the vegetation season is on the increase (vegetation K)

;;; timestep: 1 - one day (might cause some overshooting in state variables, but generally behaves pretty good), 2 - 12 hours, 24 - 1 hour timesteps (very slow)
;;;;;; Setting up model parameters and initial conditions ;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;

globals [ Sb Smax gamma rho  tauD u  beta eta sigmaM sigmaG sigmaL kappa
          lambdaM lambdaS epsilonM epsilonA epsilonC muB alphaM alphaS muS rhoS Vmax Vmin
          theta chiM chiS A Z Y impregnationDay resource resourceGrowthIncrement fetalDevDuration snowYear startSnowDay
          transitionPhase
        ]

breed [foals foal]    ;; foals are sexless, they will be assigned a sex once they wean off
breed [females female];; all non-foal females
breed [males male]

turtles-own [t_age      ;; days
             t_ageYears ;; age in years
             t_structuralMass ;;kg
             t_structuralMassIncrement ;; kg
             t_fatMass ;; kg
             t_fatIncrement ;; kg
             t_totalMass ;;kg
             t_fat2StructureRatio;; -
             t_lifespanValue     ;; -
             t_survivorshipValue ;;days
             t_resourceIntake    ;; kg
             t_maintenanceCost   ;; MJ
             t_growthCost        ;; MJ
             t_starvationMortality ;; -
             t_mother_who          ;; each individual "remembers" its mother's ID number - mainly used for code checking
             t_fetalMass4Intake    ;; fetal mass of a gravid female, kg
             t_energeticOutcomeResource ;; MJ
                        ]   

females-own [t_gestating  ;; pregnancy, true or false
             t_fetalAge  ;; days of fetus since insemination
             t_fetalMass ;; kg
             t_fetalMassIncrement ;; kg
             t_gestationCost      ;; MJ
             t_lactating  ;; lactation, 1 - false and 2 - true
             t_amountMilkProduced ;; kg
             t_lactationCost ]    ;; MJ

foals-own [ t_milkIntake ;; kg
            t_milkGain]  ;; MJ


to setup
 
  clear-all
  set Sb 25.5 ;; kg, structural mass at birth
  set Smax 280 ;; kg, maximum attainable structural weigh
  set gamma 0.0023; 1/day, growth rate in structural mass
  set rho 0.3 ;; target reserve mass as a fraction of total body Mass, W
  set tauD 60;; days, a delay in fetal development
  set u 0.32;; energy conductance rate [cm d-1], adapted from Kooijman (2000)
  set fetalDevDuration 275;; [d], fetal development duration
  set beta 100 ;; kg/ha, half saturation constant
  set eta 15 ;; /kg, steepness in satiation scaling of the intake rate 
  set sigmaM 0.6;; MJ*kg^3/4/day, scalar constant in metabolic costs
  set sigmaG 40;; MJ/kg, costs of structural growth 
  set sigmaL 3.31 ;;MJ/kg, costs of milk production (lactating females only)
  set kappa 0.0015;; /day, rate of increase in structural mass growth costs
  set lambdaM 0.03 ;; /day, maximum milk production as a fraction of total body mass, W
  set lambdaS 0.004 ;; /day, milk production decline rate
  set epsilonM 1.9 ;; MJ/kg, milk energy content
  set epsilonA 54.6 ;; MJ/kg, anabolic reserves conversion efficiency
  set epsilonC 39.3 ; MJ/kg, catabolic reserves conversion efficiency
  set muB 3.86 * 10 ^ (-5) ;; /day, background daily mortality
  set alphaM 9855 ;; d, modal age of senescence
  set alphaS 912.5 ;; d, standard deviation in age of senescence
  set muS 0.012 ;; /day, scalar constant in starvation mortality
  set rhoS 0.1 ;; reserves-mass ratio where starvation starts
  set resource 10000 ;; kg, total start-up amount of resource
  set Vmax 6000 * 750 ;; kg/ha, maximum summer resource density
  set Vmin 2000 * 750;; kg/ha, minimum winter resource density
  set theta 0.3 ;; steepness in seasonality shift
  set chiM 10.7 ;; MJ/kg, resource energy content at start of summer
  set chiS 3.5 * 10 ^ (-6) ;; rate of decline in resource energy content
  set A 750 ;; ha, total habitat area
  set Z 150 ;; d, summer duration
  set Y 364 ;; d, year length
  set impregnationDay 30 ;; females are impregnated on ca. 1st May
  set startSnowDay 305 ;; day 305 represents 1st February, the day when we assume snow cover to take place
  set transitionPhase 20 * timestep
  setupHorses
 reset-ticks
end

to setupHorses
  ;; simulation starts with 500 horses, 1:1 sex ratio, in good body condition, i.e. maximum fat-to-total mass ratio
  crt 500   [ ifelse random-float 1.0 < 0.5
             
             [set breed males
              set color brown ]
             
              [set breed females 
              set color red 
              set t_gestating false
              set t_lactating 1 ] ; 1 - false, 2 - true
              
              setxy random-pxcor random-pycor
              set t_age 1095
              set t_structuralMass ((Smax ^ (1 / 3)) - (Smax ^ (1 / 3) - Sb ^ (1 / 3)) * exp (- gamma * t_age)) ^ 3
              set t_fat2StructureRatio rho
              set t_fatMass (t_fat2StructureRatio * t_structuralMass) / (1 - t_fat2StructureRatio)
              set t_totalMass (t_structuralMass + t_fatMass)
              set t_lifespanValue random-float 1.0
             ]
end


;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;

to go
  
  ;; on day 1 in a new year, a probability of snow (GUI) will determine if this is a snow year or not  
  if dayInYear = 1[
   ifelse random-float 1.0 < snowProbability
  [set snowYear true]
  [set snowYear false]] 
  
  ;; conditions to stop the simulation 
  if not any? turtles [stop]
  if Year = 50 [stop]  
 
 
  backgroundMortality ;; survivorship curve
  ageing 
  
  ;;; calculate changes in variables;;;;
  
  ask turtles [  
  
  resourceIntake ;; includes intake of milk by foals
  structuralGrowth ;; this includes fetal growth in pregnant females (fetus is not its own entity)
  lactation   
  reserveDynamics  
  starvationMortality ]
  
  vegetation
  
  ;;; update variabels based on calculated changes
  updateVariables 
   
  reproduction
    
  tick
  
  updatePlots
  
end

to backgroundMortality
  
  ask turtles [ set t_survivorshipValue  exp (( - muB * floor t_age) - exp ( - alphaM / alphaS) * ( exp (floor t_age / alphaS) - 1))
                if  (t_lifespanValue > t_survivorshipValue)
                [ if (count my-out-links != 0)[ask out-link-neighbors [die]]
                 die]]
  
end

to ageing
  ask turtles [set t_age t_age + 1 / timestep
               set t_ageYears t_age / Y] ;; become older
  
  ask females with [t_gestating = true]
 [set t_fetalAge t_fetalAge + 1 / timestep] 
  
  ask foals with [t_age >= 364] ;; end of foal stage, they become genderized
   [ ifelse random-float 1.0 < 0.5
             [set breed males
              set color brown
              rt random 360 fd 1 ]
             [set breed females 
              set color red 
              rt random 360 fd 1
              set t_gestating false
              set t_lactating 1 
              ]
 
      ask my-in-links [die] ;; dettaching from mother, no more milk for you!      
             ]

end

;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
;;;;;;;;;;;;;;;;;;;;;;; STRUCTURAL GROWTH;;;;;;;;; ;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;

to structuralGrowth
  
  ;;; calculating the structural mass increment for each individual
              set t_structuralMassIncrement (3 * gamma * (Smax ^ (1 / 3) * t_structuralMass ^ (2 / 3) - t_structuralMass))
                 if t_structuralMassIncrement < 0
                 [set t_structuralMassIncrement 0]
                 
  ;;; pregnant females also calculate the growth of their fetus     
             if (breed = females)[
                 if t_gestating = true[
                    if t_fetalAge >= 60 [
                 let fetalAgeToStartDevelopment t_fetalAge - tauD
                 set t_fetalMassIncrement (1 / 9 * (u ^ 3) * (fetalAgeToStartDevelopment ^ 2)) ;; fetal structural Mass growth
                 set t_fetalMassIncrement t_fetalMassIncrement / 1000 ;; the equation is in g, we need kgs
               ]]]
  
  
end

;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
;;;;;;;;;;;;;;;;;;;;;;;  LACTATION ;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;  

to lactation
  ;; only lactating females produce milk
  
  if breed = foals [set t_milkIntake 0]  ;; resetting the milk intake from the previous time step
  if breed = females [
    if t_lactating = 2 [
       set t_amountMilkProduced 0]]      
  
   if breed = females [
      if count my-out-links != 0[   
      ;; females milk production is a function of the foal's fat mass - establishing a link with the foal, we ensure that take only their own foals into account
                let my_foalAge floor [t_age] of one-of out-link-neighbors                
                let my_foalTotalMass [t_totalMass] of one-of out-link-neighbors              
                let my_foalFatMass [t_fatMass] of one-of out-link-neighbors
                
                set t_amountMilkProduced (lambdaM * t_totalMass * exp(- (lambdaS) * my_foalAge)) / (1 + exp(- eta * ( (rho * my_foalTotalMass) - my_foalFatMass)))
               
                ]]              
                
end

;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
;;;;;;;;;;;;;;;;;;;;;;; RESOURCE INTAKE ;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
to resourceIntake
  
  set t_fetalMass4Intake 0  
  if breed = females [
    if t_gestating = true [
      set t_fetalMass4Intake t_fetalMass]];; females should eat more based on the mass of their fetus
  
  ;;; if there is snow cover, intake will be 0 for a period defined on GUI
  ifelse (snowYear = true and dayInYear >= startSnowDay and dayInYear < startSnowDay + snowCoverLength)
  [set t_resourceIntake 0] 
  [set t_resourceIntake alpha * ((t_structuralMass + t_fetalMass4Intake) ^ (3 / 4)) * (resource / (beta + resource)) * (1 / (1 + exp (- eta * ((rho * t_totalMass) - t_fatMass))))
  
  set resource resource - t_resourceIntake / timestep;; each turtle subtracts the amount it ate from the whole pool of resource
  ]
end

;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
;;;;;;;;;;;;;;;;;;;;;;; RESERVE DYNAMICS ;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;

to reserveDynamics

;;;;;;;;;;;;; FOALS ;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
     
              if breed = foals [
    ;; energy gains
                  set t_milkIntake mean [t_amountMilkProduced] of in-link-neighbors  
       
                  set t_milkGain (epsilonM * t_milkIntake)
                  
                  set t_energeticOutcomeResource (resourceEnergy * t_resourceIntake) ;; in MegaJoules
                 
    ;; energy costs
                  set t_maintenanceCost (sigmaM * (t_totalMass ^ ( 3 / 4 )));; in MJ
               
                  
                  let costOfGrowth  sigmaG * exp (kappa * t_age)      
                  set t_growthCost ( costOfGrowth * t_structuralMassIncrement);; in MJ  
      

                  let l_energyDynamics (t_energeticOutcomeResource + t_milkGain - t_maintenanceCost - t_growthCost)  ;; eq. 11 ODD, basic energy dynamics gains - costs
                  
            ;; if gains exceed costs, extra fat mass is formed, if not, fat mass is used for satisfying costs      
                  ifelse l_energyDynamics > 0
                  [ set t_fatIncrement l_energyDynamics / epsilonA]
                  [ set t_fatIncrement l_energyDynamics / epsilonC]
                   
                  ]

;;;;;;;;;;;;;;;;;;;;;;;;;;;; ADULTS ;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
  
   if breed = females [
  ;; energy gains
               set t_energeticOutcomeResource (resourceEnergy * t_resourceIntake) ;; in MegaJoules
               ;print energeticOutcomeResource
  ;; energy costs           
               let costOfGrowth  sigmaG * exp (kappa * t_age)      
               set t_growthCost ( costOfGrowth * t_structuralMassIncrement);; in MJ  
         
  
               ifelse (t_lactating = 2)   
               [set t_lactationCost (sigmaL * t_amountMilkProduced )];; only for lactating females
               [set t_lactationCost 0]
  
              ;; gestation and maintenance costs depend on whether females are gestating or not
               ifelse (t_gestating = true) 
               [set t_gestationCost t_fetalMassIncrement * sigmaG                        ;; fixed cost of fetal growth, unlike for hatched individuals
                set t_maintenanceCost (sigmaM * (t_totalMass + t_fetalMass) ^ ( 3 / 4 ))] 
               
               [set t_gestationCost 0
                set t_maintenanceCost (sigmaM * (t_totalMass) ^ ( 3 / 4 ))] 
  
               let l_energyDynamics t_energeticOutcomeResource - t_maintenanceCost - t_growthCost - t_lactationCost - t_gestationCost ;; eq. 11 ODD, basic energy dynamics
   ;; if gains exceed costs, extra fat mass is formed, if not, fat mass is used for satisfying costs   
               ifelse l_energyDynamics > 0
               [ set t_fatIncrement l_energyDynamics / epsilonA]
               [ set t_fatIncrement l_energyDynamics / epsilonC]
    
                 ]
  
        if breed = males [
  ;; energy gains
               set t_energeticOutcomeResource (resourceEnergy * t_resourceIntake) ;; in MegaJoules
  ;; energy costs
               set t_maintenanceCost (sigmaM * (t_totalMass ^ ( 3 / 4 )));; in MJ
             
              let costOfGrowth  sigmaG * exp (kappa * t_age)      
              set t_growthCost ( costOfGrowth * t_structuralMassIncrement);; in MJ  
            
                  
               let l_energyDynamics t_energeticOutcomeResource - t_maintenanceCost - t_growthCost ;; eq. 11 ODD, basic energy dynamics
     
     ;; if gains exceed costs, extra fat mass is formed, if not, fat mass is used for satisfying costs         
               ifelse l_energyDynamics > 0
                [ set t_fatIncrement l_energyDynamics / epsilonA]
                [ set t_fatIncrement l_energyDynamics / epsilonC]
               ]  
end

;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
;;;;;;;;;;;;;;;;;;;;;;; VEGETATION DYNAMICS ;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
to vegetation  
    ;;; if there is snow cover, respource growth will be 0 for a period defined on GUI
  ifelse (snowYear = true and dayInYear >= startSnowDay and dayInYear < startSnowDay + snowCoverLength)
  [set resourceGrowthIncrement 0]  
  [set resourceGrowthIncrement ( r * (resourceK - resource))] 
end

;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
;;;;;;;;;;;;;;;;;;;;;;; STARVATION MORTALITY ;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;           
to starvationMortality
             if t_fat2StructureRatio < rhoS [
             
               set t_starvationMortality exp(- muS * (rhoS * (t_totalMass / t_fatMass) - 1))
     
                if random-float 1.0 < t_starvationMortality
                  [ if (count my-out-links != 0)[ask out-link-neighbors [die]]
                    die]] 
   

end  

;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
;;;;;;;;;;;;;;;;;;;;;;; UPDATE ALL VARIABLES ;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;    

to updateVariables
  
  ask females with [t_gestating = true] [
      set t_fetalMass t_fetalMass + (t_fetalMassIncrement / timestep)]                                               
  
  ask turtles [
      set t_structuralMass t_structuralMass + (t_structuralMassIncrement / timestep)
                   
      set t_fatMass t_fatMass + (t_fatIncrement / timestep)
                
      set t_fat2StructureRatio t_fatMass / t_totalMass   
      set t_totalMass t_structuralMass + t_fatMass]
  
     
     set resource resource + (resourceGrowthIncrement / timestep) 
     if resource <= 0 [set resource 0]
 end




;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
;;;;;;;;;;;;;;;;;;;;;;; REPRODUCTION ;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;

to reproduction
  
  ;; first the gestating females give birth to foals (only 1 per female), on 1st April 
  
  if round dayInYear = 1 [
   ask females with [t_gestating = true] [
    
   hatch-foals 1 [set color yellow
                  rt random 360 fd 1
                  set t_structuralMass Sb
                  set t_age 1
                  set t_ageYears t_age / Y
                  set t_lifespanValue random-float 1.0
                  set t_fat2StructureRatio [t_fat2StructureRatio] of myself         
                  set t_fatMass (t_fat2StructureRatio * t_structuralMass) / (1 - t_fat2StructureRatio);; amount of fat the same as its mother's
                  set t_totalMass (t_structuralMass + t_fatMass) 
                  set t_maintenanceCost 0
                  set t_growthCost 0 
                  set t_resourceIntake 0
                  create-link-from myself           ;; link between the mother and her foal is created
                  set t_mother_who [who] of myself ] 
   
   set t_fatMass t_fatMass - mean [t_fatMass] of out-link-neighbors ;; the amount of fat mass given to the foal is deducted from its mother
   set t_lactating 2
   set t_gestating false
   set t_fetalMass 0
   set t_fetalAge 0
   set t_fetalMassIncrement 0
          ]]
  
  ;; then all females have a 0.56 probability of getting pregnant again, on ca 1st May (day 30)
  
  if dayInYear = impregnationDay [
    ask females with [t_ageYears >= 3]
    [ if random-float 1.0 <= 0.56
      [ set t_gestating true
        set t_fetalAge 1
        set t_fetalMass 0]
  ]]
  
end


;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
;;;;;;;;;;;;;;;;;;;;;;; PLOTTING ;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;

to updatePlots
  set-current-plot "Horse population"
  set-current-plot-pen "Foals"
  set-plot-pen-interval 1 / timestep
  plot count turtles with [t_ageYears < 1 ]
  set-current-plot-pen "Yearlings"
  set-plot-pen-interval 1 / timestep
  plot count turtles with [t_ageYears >= 1 and t_ageYears < 2]
  set-current-plot-pen "Adults"
  set-plot-pen-interval 1 / timestep
  plot count turtles with [t_ageYears > 2]
  set-current-plot-pen "Total"
  set-plot-pen-interval 1 / timestep
  plot count turtles
  
  
  set-current-plot "Horse biomass"
  if any? turtles with [t_ageYears >= 3][
     
  set-current-plot-pen "Structural > 3 yrs"
  set-plot-pen-interval 1 / timestep 
  plot mean [t_structuralMass] of turtles with [t_ageYears >= 3]
  set-current-plot-pen "Fat > 3 yrs"
  set-plot-pen-interval 1 / timestep
  plot mean [t_fatMass] of turtles with [t_ageYears >= 3]]
  
 
  
  if any? turtles with [t_ageYears <= 3][
  set-current-plot-pen "Structural < 3 yrs" 
  set-plot-pen-interval 1 / timestep
  plot mean [t_structuralMass] of turtles with [t_ageYears <= 3]
  set-current-plot-pen "Fat < 3 yrs"
  set-plot-pen-interval 1 / timestep
  plot mean [t_fatMass] of turtles with [t_ageYears <= 3]
  ]
  

  
  
  set-current-plot "Vegetation"
  set-current-plot-pen "Vegetation"
  set-plot-pen-interval 1 / timestep
  plot resource
  set-current-plot-pen "Veg K"
  set-plot-pen-interval 1 / timestep
  plot resourceK
  
  
  set-current-plot "Fat-to-total mass ratio"
  ifelse any? turtles with [t_ageYears >= 3]
  [set-current-plot-pen "Adults"
   set-plot-pen-interval 1 / timestep
   plot mean [t_fat2StructureRatio] of turtles with [t_ageYears >= 3]]
    [set-current-plot-pen "Adults"
     set-plot-pen-interval 1 / timestep
     plot 0.3]
  
  ifelse any? turtles with [t_ageYears >= 2 and t_ageYears < 3]
  [ set-current-plot-pen "2 yr olds"
    set-plot-pen-interval 1 / timestep
    plot mean [t_fat2StructureRatio] of turtles with [t_ageYears >= 2 and t_ageYears < 3]]
  [ set-current-plot-pen "2 yr olds"
    set-plot-pen-interval 1 / timestep
    plot 0.3]
  
  ifelse any? turtles with [t_ageYears >= 1 and t_ageYears < 2]
  [set-current-plot-pen "Yearlings"
   set-plot-pen-interval 1 / timestep
   plot mean [t_fat2StructureRatio] of turtles with [t_ageYears >= 1 and t_ageYears < 2]]
  [set-current-plot-pen "Yearlings"
   set-plot-pen-interval 1 / timestep
   plot 0.3]
  
  ifelse any? turtles with [t_ageYears < 1]
  [set-current-plot-pen "Foals"
   set-plot-pen-interval 1 / timestep
   plot mean [t_fat2StructureRatio] of turtles with [t_ageYears < 1]]
  [set-current-plot-pen "Foals"
   set-plot-pen-interval 1 / timestep
   plot 0.3]

  set-current-plot-pen "Starvation threshold" 
  set-plot-pen-interval 1 / timestep
  plotxy 0 0.1
  plotxy plot-x-max 0.1 
  
  
  
  set-current-plot "Population age structure"
  set-histogram-num-bars 50
  set-plot-pen-interval 1 / timestep
  if any? turtles[
  histogram [t_ageYears]  of turtles]
  
  
end

;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
;;;;;;;;;;; REPORTERS ;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;

to-report year
           report floor ticks / (Y * timestep)
end

to-report dayInYear
          report (ticks / timestep) mod Y                 ;;365 days a year
end



to-report resourceK
   
   ifelse ( dayInYear <= 150 )
 
        [report  Vmax + ((Vmax - Vmin) / 2)  * ( 1 + ( sin (( 180 * dayInYear)/ Z)) ^ theta ) ]
        
        [report  Vmax + ((Vmax - Vmin) / 2) * ( 1 - ( sin ( 180 * (dayInYear - Z)/ ( Y - Z)) ^ theta ))]

   report resourceK
end  

to-report resourceEnergy
  report  chiM * (1 - chiS * (tauS) ^ 2) ^ 0.5
end

to-report tauS
  report dayInYear mod Y
end 

to-report juvenileFat2Weight
  ifelse (count turtles with [t_age < 1095] != 0)
  [report mean [t_fat2StructureRatio] of turtles with [t_age < 1095]]
  [report "NA"]
end
@#$#@#$#@
GRAPHICS-WINDOW
1091
16
1336
246
16
16
6.03030303030303
1
10
1
1
1
0
1
1
1
-16
16
-16
16
1
1
1
ticks
30.0

BUTTON
4
16
67
49
NIL
setup
NIL
1
T
OBSERVER
NIL
NIL
NIL
NIL
1

BUTTON
3
53
66
86
NIL
go
T
1
T
OBSERVER
NIL
NIL
NIL
NIL
1

PLOT
141
10
1083
348
Horse population
Time [days]
Population count [ind]
0.0
10.0
0.0
10.0
true
true
"" ""
PENS
"Foals" 1.0 0 -2674135 true "" ""
"Yearlings" 1.0 0 -7500403 true "" ""
"Adults" 1.0 0 -16777216 true "" ""
"Total" 1.0 0 -15040220 true "" ""

MONITOR
665
14
723
59
Females
count females
17
1
11

MONITOR
726
14
783
59
Males
count males
17
1
11

PLOT
139
622
1088
850
Horse biomass
Time [days]
Biomass [kg]
0.0
10.0
0.0
10.0
true
true
"" ""
PENS
"Structural > 3 yrs" 1.0 0 -6995700 true "" ""
"Fat > 3 yrs" 1.0 0 -817084 true "" ""
"Structural < 3 yrs" 1.0 0 -15302303 true "" ""
"Fat < 3 yrs" 1.0 0 -11085214 true "" ""

MONITOR
4
99
81
144
Day in year
dayInYear + 1
17
1
11

MONITOR
3
147
80
192
Year
year
17
1
11

PLOT
139
350
1086
620
Vegetation
Time [days]
Vegetation total biomass [kg]
0.0
10.0
0.0
8140000.0
true
true
"" ""
PENS
"Vegetation" 1.0 0 -16777216 true "" ""
"Veg K" 1.0 0 -10899396 true "" ""

PLOT
1091
251
1820
618
Fat-to-total mass ratio
Time [days]
[-]
0.0
10.0
0.05
0.35
true
true
"" ""
PENS
"Adults" 1.0 0 -16777216 true "" ""
"2 yr olds" 1.0 0 -955883 true "" ""
"Yearlings" 1.0 0 -7500403 true "" ""
"Foals" 1.0 0 -2674135 true "" ""
"Starvation threshold" 1.0 0 -14730904 true "" ""

PLOT
1307
10
1817
246
Population age structure
Age classes
Frequency
0.0
10.0
0.0
10.0
true
false
"" ""
PENS
"default" 1.0 1 -16777216 true "" ""

MONITOR
897
13
1002
58
Population count
count turtles
17
1
11

MONITOR
787
14
837
59
Foals
count foals
17
1
11

PLOT
1090
619
1821
847
Foals
NIL
[kg]
0.0
10.0
0.0
0.5
true
true
"" ""
PENS
"Milk intake" 1.0 0 -5825686 true "set-plot-pen-interval 1 / timestep" "if any? foals [plot mean [t_milkIntake] of foals]"
"Vegetation intake" 1.0 0 -13840069 true "set-plot-pen-interval 1 / timestep" "if any? foals [plot mean [t_resourceIntake] of foals]"

BUTTON
67
54
140
87
Go for 1 day
repeat timestep [go]
NIL
1
T
OBSERVER
NIL
NIL
NIL
NIL
1

SLIDER
2
555
136
588
SnowProbability
SnowProbability
0
1
0
0.1
1
NIL
HORIZONTAL

SLIDER
1
592
136
625
snowCoverLength
snowCoverLength
0
60
20
5
1
NIL
HORIZONTAL

TEXTBOX
11
491
134
560
Model variant - extreme weather (no resource growth, no resource intake)
11
0.0
1

MONITOR
1
629
135
674
Is there snow this year?
snowYear
17
1
11

INPUTBOX
30
224
89
284
omikron
0.56
1
0
Number

INPUTBOX
32
308
82
368
r
0.0020
1
0
Number

INPUTBOX
26
403
91
463
alpha
0.156
1
0
Number

TEXTBOX
6
204
156
222
Impregnation probability
11
0.0
1

TEXTBOX
2
289
152
307
Resource growth rate [d^-1]
11
0.0
1

TEXTBOX
6
368
156
396
Maximum intake rate [kg kg^3/4d^1]
11
0.0
1

INPUTBOX
22
701
84
761
timestep
2
1
0
Number

TEXTBOX
7
768
123
819
1 - one day, 2 - 12 hours, 24 - 1 hour timesteps (very slow)
11
0.0
1

@#$#@#$#@
## WHAT IS IT?

The main purpose of the model is to explore how resource competition shapes individual life history and its feedback on dynamics of a horse population in a seasonal environment. Specifically, the model investigates the link between prescribed growth in body size, population dynamics and density dependence through population feedback on available resources. 

The model parameters are based on the Konik horse population is Oostvaardersplassen in The Netherlands. This model version is non-spatial. 

## HOW IT WORKS

For a detailed model description see the ODD (KonikIBM_ODD).

## THINGS TO TRY

Model users can test how different parameters affect individual life history and population dynamics. Specifically, three global parameters are placed on GUI:
1) omikron - impregnation probability for each females older than 3 years
2) r - resource growth rate, d-1
3) alpha - maximum resource intake rate, kg kg^3/4 d^1

**Model variant – extreme weather**

To test the effects of extreme weather conditions, in the form of snow cover, on individual bioenergetics and population dynamics, we implemented a probability that snow cover will occur in a given year and the length of the snow cover period. Both are defined by the model user on GUI (snowProbability and snowCoverLength).

During the snow cover period, there is no resource growth or resource intake by all grazing individuals in the population. All the baseline model processes are exactly the same as previously described (see ODD for a detailed model description). 


## CREDITS AND REFERENCES

This model is a NetLogo implementation of the model published in:

De Roos, A. M.; Galic, N.; Heesterbeek, H., How resource competition shapes individual life history for nonplastic growth: Ungulates in seasonal food environments. Ecology 2009, 90, (4), 945-960.
@#$#@#$#@
default
true
0
Polygon -7500403 true true 150 5 40 250 150 205 260 250

airplane
true
0
Polygon -7500403 true true 150 0 135 15 120 60 120 105 15 165 15 195 120 180 135 240 105 270 120 285 150 270 180 285 210 270 165 240 180 180 285 195 285 165 180 105 180 60 165 15

arrow
true
0
Polygon -7500403 true true 150 0 0 150 105 150 105 293 195 293 195 150 300 150

box
false
0
Polygon -7500403 true true 150 285 285 225 285 75 150 135
Polygon -7500403 true true 150 135 15 75 150 15 285 75
Polygon -7500403 true true 15 75 15 225 150 285 150 135
Line -16777216 false 150 285 150 135
Line -16777216 false 150 135 15 75
Line -16777216 false 150 135 285 75

bug
true
0
Circle -7500403 true true 96 182 108
Circle -7500403 true true 110 127 80
Circle -7500403 true true 110 75 80
Line -7500403 true 150 100 80 30
Line -7500403 true 150 100 220 30

butterfly
true
0
Polygon -7500403 true true 150 165 209 199 225 225 225 255 195 270 165 255 150 240
Polygon -7500403 true true 150 165 89 198 75 225 75 255 105 270 135 255 150 240
Polygon -7500403 true true 139 148 100 105 55 90 25 90 10 105 10 135 25 180 40 195 85 194 139 163
Polygon -7500403 true true 162 150 200 105 245 90 275 90 290 105 290 135 275 180 260 195 215 195 162 165
Polygon -16777216 true false 150 255 135 225 120 150 135 120 150 105 165 120 180 150 165 225
Circle -16777216 true false 135 90 30
Line -16777216 false 150 105 195 60
Line -16777216 false 150 105 105 60

car
false
0
Polygon -7500403 true true 300 180 279 164 261 144 240 135 226 132 213 106 203 84 185 63 159 50 135 50 75 60 0 150 0 165 0 225 300 225 300 180
Circle -16777216 true false 180 180 90
Circle -16777216 true false 30 180 90
Polygon -16777216 true false 162 80 132 78 134 135 209 135 194 105 189 96 180 89
Circle -7500403 true true 47 195 58
Circle -7500403 true true 195 195 58

circle
false
0
Circle -7500403 true true 0 0 300

circle 2
false
0
Circle -7500403 true true 0 0 300
Circle -16777216 true false 30 30 240

cow
false
0
Polygon -7500403 true true 200 193 197 249 179 249 177 196 166 187 140 189 93 191 78 179 72 211 49 209 48 181 37 149 25 120 25 89 45 72 103 84 179 75 198 76 252 64 272 81 293 103 285 121 255 121 242 118 224 167
Polygon -7500403 true true 73 210 86 251 62 249 48 208
Polygon -7500403 true true 25 114 16 195 9 204 23 213 25 200 39 123

cylinder
false
0
Circle -7500403 true true 0 0 300

dot
false
0
Circle -7500403 true true 90 90 120

face happy
false
0
Circle -7500403 true true 8 8 285
Circle -16777216 true false 60 75 60
Circle -16777216 true false 180 75 60
Polygon -16777216 true false 150 255 90 239 62 213 47 191 67 179 90 203 109 218 150 225 192 218 210 203 227 181 251 194 236 217 212 240

face neutral
false
0
Circle -7500403 true true 8 7 285
Circle -16777216 true false 60 75 60
Circle -16777216 true false 180 75 60
Rectangle -16777216 true false 60 195 240 225

face sad
false
0
Circle -7500403 true true 8 8 285
Circle -16777216 true false 60 75 60
Circle -16777216 true false 180 75 60
Polygon -16777216 true false 150 168 90 184 62 210 47 232 67 244 90 220 109 205 150 198 192 205 210 220 227 242 251 229 236 206 212 183

fish
false
0
Polygon -1 true false 44 131 21 87 15 86 0 120 15 150 0 180 13 214 20 212 45 166
Polygon -1 true false 135 195 119 235 95 218 76 210 46 204 60 165
Polygon -1 true false 75 45 83 77 71 103 86 114 166 78 135 60
Polygon -7500403 true true 30 136 151 77 226 81 280 119 292 146 292 160 287 170 270 195 195 210 151 212 30 166
Circle -16777216 true false 215 106 30

flag
false
0
Rectangle -7500403 true true 60 15 75 300
Polygon -7500403 true true 90 150 270 90 90 30
Line -7500403 true 75 135 90 135
Line -7500403 true 75 45 90 45

flower
false
0
Polygon -10899396 true false 135 120 165 165 180 210 180 240 150 300 165 300 195 240 195 195 165 135
Circle -7500403 true true 85 132 38
Circle -7500403 true true 130 147 38
Circle -7500403 true true 192 85 38
Circle -7500403 true true 85 40 38
Circle -7500403 true true 177 40 38
Circle -7500403 true true 177 132 38
Circle -7500403 true true 70 85 38
Circle -7500403 true true 130 25 38
Circle -7500403 true true 96 51 108
Circle -16777216 true false 113 68 74
Polygon -10899396 true false 189 233 219 188 249 173 279 188 234 218
Polygon -10899396 true false 180 255 150 210 105 210 75 240 135 240

house
false
0
Rectangle -7500403 true true 45 120 255 285
Rectangle -16777216 true false 120 210 180 285
Polygon -7500403 true true 15 120 150 15 285 120
Line -16777216 false 30 120 270 120

leaf
false
0
Polygon -7500403 true true 150 210 135 195 120 210 60 210 30 195 60 180 60 165 15 135 30 120 15 105 40 104 45 90 60 90 90 105 105 120 120 120 105 60 120 60 135 30 150 15 165 30 180 60 195 60 180 120 195 120 210 105 240 90 255 90 263 104 285 105 270 120 285 135 240 165 240 180 270 195 240 210 180 210 165 195
Polygon -7500403 true true 135 195 135 240 120 255 105 255 105 285 135 285 165 240 165 195

line
true
0
Line -7500403 true 150 0 150 300

line half
true
0
Line -7500403 true 150 0 150 150

pentagon
false
0
Polygon -7500403 true true 150 15 15 120 60 285 240 285 285 120

person
false
0
Circle -7500403 true true 110 5 80
Polygon -7500403 true true 105 90 120 195 90 285 105 300 135 300 150 225 165 300 195 300 210 285 180 195 195 90
Rectangle -7500403 true true 127 79 172 94
Polygon -7500403 true true 195 90 240 150 225 180 165 105
Polygon -7500403 true true 105 90 60 150 75 180 135 105

plant
false
0
Rectangle -7500403 true true 135 90 165 300
Polygon -7500403 true true 135 255 90 210 45 195 75 255 135 285
Polygon -7500403 true true 165 255 210 210 255 195 225 255 165 285
Polygon -7500403 true true 135 180 90 135 45 120 75 180 135 210
Polygon -7500403 true true 165 180 165 210 225 180 255 120 210 135
Polygon -7500403 true true 135 105 90 60 45 45 75 105 135 135
Polygon -7500403 true true 165 105 165 135 225 105 255 45 210 60
Polygon -7500403 true true 135 90 120 45 150 15 180 45 165 90

sheep
false
0
Rectangle -7500403 true true 151 225 180 285
Rectangle -7500403 true true 47 225 75 285
Rectangle -7500403 true true 15 75 210 225
Circle -7500403 true true 135 75 150
Circle -16777216 true false 165 76 116

square
false
0
Rectangle -7500403 true true 30 30 270 270

square 2
false
0
Rectangle -7500403 true true 30 30 270 270
Rectangle -16777216 true false 60 60 240 240

star
false
0
Polygon -7500403 true true 151 1 185 108 298 108 207 175 242 282 151 216 59 282 94 175 3 108 116 108

target
false
0
Circle -7500403 true true 0 0 300
Circle -16777216 true false 30 30 240
Circle -7500403 true true 60 60 180
Circle -16777216 true false 90 90 120
Circle -7500403 true true 120 120 60

tree
false
0
Circle -7500403 true true 118 3 94
Rectangle -6459832 true false 120 195 180 300
Circle -7500403 true true 65 21 108
Circle -7500403 true true 116 41 127
Circle -7500403 true true 45 90 120
Circle -7500403 true true 104 74 152

triangle
false
0
Polygon -7500403 true true 150 30 15 255 285 255

triangle 2
false
0
Polygon -7500403 true true 150 30 15 255 285 255
Polygon -16777216 true false 151 99 225 223 75 224

truck
false
0
Rectangle -7500403 true true 4 45 195 187
Polygon -7500403 true true 296 193 296 150 259 134 244 104 208 104 207 194
Rectangle -1 true false 195 60 195 105
Polygon -16777216 true false 238 112 252 141 219 141 218 112
Circle -16777216 true false 234 174 42
Rectangle -7500403 true true 181 185 214 194
Circle -16777216 true false 144 174 42
Circle -16777216 true false 24 174 42
Circle -7500403 false true 24 174 42
Circle -7500403 false true 144 174 42
Circle -7500403 false true 234 174 42

turtle
true
0
Polygon -10899396 true false 215 204 240 233 246 254 228 266 215 252 193 210
Polygon -10899396 true false 195 90 225 75 245 75 260 89 269 108 261 124 240 105 225 105 210 105
Polygon -10899396 true false 105 90 75 75 55 75 40 89 31 108 39 124 60 105 75 105 90 105
Polygon -10899396 true false 132 85 134 64 107 51 108 17 150 2 192 18 192 52 169 65 172 87
Polygon -10899396 true false 85 204 60 233 54 254 72 266 85 252 107 210
Polygon -7500403 true true 119 75 179 75 209 101 224 135 220 225 175 261 128 261 81 224 74 135 88 99

wheel
false
0
Circle -7500403 true true 3 3 294
Circle -16777216 true false 30 30 240
Line -7500403 true 150 285 150 15
Line -7500403 true 15 150 285 150
Circle -7500403 true true 120 120 60
Line -7500403 true 216 40 79 269
Line -7500403 true 40 84 269 221
Line -7500403 true 40 216 269 79
Line -7500403 true 84 40 221 269

x
false
0
Polygon -7500403 true true 270 75 225 30 30 225 75 270
Polygon -7500403 true true 30 75 75 30 270 225 225 270

@#$#@#$#@
NetLogo 5.0.3
@#$#@#$#@
@#$#@#$#@
@#$#@#$#@
@#$#@#$#@
@#$#@#$#@
default
0.0
-0.2 0 1.0 0.0
0.0 1 1.0 0.0
0.2 0 1.0 0.0
link direction
true
0
Line -7500403 true 150 150 90 180
Line -7500403 true 150 150 210 180

@#$#@#$#@
0
@#$#@#$#@
